#
# OrderModel.rb
#
# Copyright 2005 Tero Hasu
#

module ChaosOrder
  class OrderModel
    class Item
      attr_reader :page, :publisher, :code, :title, :cost
      attr_accessor :qty

      def initialize(item)
        @page, @publisher, @code, @title, @cost = item
        @qty = 0
      end

      def to_s
        [qty.to_s, page, publisher.to_s, code, title, cost].join(" ")
      end

      def to_order_line
        [qty.to_s, code, title, cost].join("\t")
      end
    end

    attr_reader :items

    def initialize(items)
      @items = items.collect {|x| Item.new(x)}
    end

    def each
      for elem in @items
        yield elem
      end
    end
    include Enumerable

    def select_by_page page
      select do |elem|
        elem.page.casecmp(page) == 0
      end
    end

    def select_by_title pattern
      re = Regexp.new(Regexp.escape(pattern), Regexp::IGNORECASE)
      select do |elem|
        elem.title =~ re
      end
    end

    def select_ordered
      select do |elem|
        elem.qty > 0
      end
    end

    def total_qty
      select_ordered.inject(0) do |qty, elem|
        qty + elem.qty
      end
    end

    def total_cost
      tcost = 0.0
      pis = ''
      for elem in select_ordered
        ecost = elem.cost
        if ecost == ''
          pis += '+unknown'
        elsif ecost =~ /^[0-9\.]+$/
          tcost += ecost.to_f
        else
          pis += "+" + ecost
        end
      end
      return '$' + format("%.2f", tcost) + pis
    end
  end
end

if $0 == __FILE__
  require 'pp'

  $LOAD_PATH.push(File.dirname(File.expand_path(__FILE__)))
  file = ARGV.shift || '../forms/form-2004_03.ord'
  require 'ImportForm'
  data = ChaosOrder::get_form_data_from_file(file)
  require 'ParseForm'
  parser = ChaosOrder::OrderFormParser.new
  parser.parse(data)
  model = ChaosOrder::OrderModel.new(parser.items)
  #pp model
  #pp model.select_by_page('44')
  #pp model.select_by_title('wolverine')
  model.items[5].qty = 1
  model.items[10].qty = 2
  pp model.select_ordered
  pp model.total_cost
end
